use std::vec::Drain;

use ::child_task::ChildTask;
use ::monitor::Monitor;
use ::task::Task;
use ::task_container::TaskContainer;
use ::task_state_machine::TaskStateMachine;



/// This structure contains the child and sibling Tasks
/// that a task has spawned while it was being processed.
pub struct Spawner
{
   /// The child tasks that have been spawned.
   ///
   /// Please use the functions so that they
   /// can be properly wrapped.
   child_tasks: Vec<Box<TaskStateMachine>>,

   /// The sibling tasks that have been spawned.
   ///
   /// Please use the functions so that they
   /// can be properly wrapped.
   sibling_tasks: Vec<Box<TaskStateMachine>>
}



impl Spawner
{
   /// Creates a new Spawner structure.
   pub fn new() -> Spawner
   {
      // Create the new Spawner.
      Spawner
      {
         child_tasks: Vec::new(),
         sibling_tasks: Vec::new()
      }
   }

   /// Spawn a new child task.
   pub fn spawn_child_task<T>(&mut self, child: T)
      where T: Task + 'static
   {
      let child_task: ChildTask<T>;
      let container: TaskContainer<ChildTask<T>>;

      child_task = ChildTask::new(child);
      container = TaskContainer::new(child_task);
      self.child_tasks.push(Box::new(container));
   }

   /// Spawn a new sibling task.
   pub fn spawn_sibling_task<T>(&mut self, sibling: T)
      where T: Task + 'static
   {
      let container: TaskContainer<T>;

      container = TaskContainer::new(sibling);
      self.sibling_tasks.push(Box::new(container));
   }

   /// Returns true if child tasks were spawned; Otherwise, false.
   pub fn has_child_tasks(&self) -> bool
   {
      !self.child_tasks.is_empty()
   }

   /// Returns true if sibling tasks were spawned; Otherwise, false.
   pub fn has_sibling_tasks(&self) -> bool
   {
      !self.sibling_tasks.is_empty()
   }

   /// Get a Vector of the spawned child tasks.
   ///
   /// This will remove all the tasks from this spawner.
   pub fn retrieve_child_tasks(&mut self) -> Vec<Box<TaskStateMachine>>
   {
      self.child_tasks.drain(..).collect()
   }

   /// Get a Vector of the spawned sibling tasks.
   ///
   /// This will remove all the tasks from this spawner.
   pub fn retrieve_sibling_tasks(&mut self) -> Vec<Box<TaskStateMachine>>
   {
      self.sibling_tasks.drain(..).collect()
   }
}
