use std::cmp::Ordering;

use ::mortality::Mortality;
use ::priority::Priority;
use ::spawner::Spawner;



pub trait Task: Send + Sync
{
   /// Get the name for this Task.
   /// This does not need to be unique.
   fn get_name(&self) -> &str;

   /// Get whether or not that this task
   /// should be reset and rerun after
   /// it has finished.
   fn get_mortality(&self) -> Mortality;

   /// Get the Priority level to run
   /// this task at.
   fn get_priority(&self) -> Priority;


   /// Reset the task to its initial state.
   fn reset(&mut self);

   /// Process a single iteration of this Task.
   /// This will continue to be called until
   /// the task has been marked as finished.
   ///
   /// This returns true, if the process is complete;
   /// Otherwise, it returns false.
   ///
   /// The given Spawner can be used to spawn child
   /// or sibling tasks.
   fn process(&mut self, spawner: &mut Spawner) -> bool;
}



impl PartialEq for Task
{
   fn eq(&self, other: &Self) -> bool
   {
      // Compare them first by priority, then by name.
      match self.get_priority().cmp(&other.get_priority())
      {
         Ordering::Equal =>
         {
            self.get_name().to_string().eq(&other.get_name().to_string())
         }

         _ =>
         {
            false
         }
      }
   }
}

impl Eq for Task
{
}

impl PartialOrd for Task
{
   fn partial_cmp(&self, other: &Self) -> Option<Ordering>
   {
      Some(self.cmp(other))
   }
}

impl Ord for Task
{
   fn cmp(&self, other: &Self) -> Ordering
   {
      // Compare them first by priority, then by name.
      match self.get_priority().cmp(&other.get_priority())
      {
         Ordering::Less =>
         {
            Ordering::Less
         }

         Ordering::Equal =>
         {
            self.get_name().to_string().cmp(&other.get_name().to_string())
         }

         Ordering::Greater =>
         {
            Ordering::Greater
         }
      }
   }
}
